package com.fr.plugin.xxxx.cqjg.jump.handle;

import com.fr.base.ServerConfig;
import com.fr.base.TemplateUtils;
import com.fr.data.NetworkHelper;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.User;
import com.fr.decision.config.FSConfig;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.mobile.terminal.TerminalHandler;
import com.fr.decision.webservice.CrossDomainResponse;
import com.fr.decision.webservice.bean.authentication.LoginClientBean;
import com.fr.decision.webservice.exception.user.UserNotExistException;
import com.fr.decision.webservice.utils.DecisionStatusService;
import com.fr.log.FineLoggerFactory;
import com.fr.security.JwtUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.stable.web.Device;
import com.fr.store.Converter;
import com.fr.third.com.lowagie.text.pdf.BadPasswordException;
import com.fr.third.org.apache.commons.codec.digest.DigestUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;


/**
 * @author fr.open
 * @Date 2021/08/17
 */
public class TokenHandler extends BaseHttpHandler {

    @Override
    public RequestMethod getMethod() {
        return RequestMethod.GET;
    }

    @Override
    public String getPath() {
        return "/token";
    }

    @Override
    public boolean isPublic() {
        return true;
    }


    @Override
    public void handle(HttpServletRequest req, HttpServletResponse res) {
        CrossDomainResponse domainResponse = null;
        try {
            String origin = NetworkHelper.getOriginalURL(req);
            String uri = req.getRequestURI();
            String render = TemplateUtils.render(origin.replace(uri, "${fineServletURL}"));
            String username = WebUtils.getHTTPRequestParameter(req, "fine_username");
            String password = WebUtils.getHTTPRequestParameter(req, "fine_password");
            String validity = WebUtils.getHTTPRequestParameter(req, "validity");
            User user = AuthorityContext.getInstance().getUserController().findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("userName", username)));
            if (user == null) {
                throw new UserNotExistException();
            }
            if (!StringUtils.equals(DigestUtils.sha256Hex(password), user.getPassword())) {
                throw new BadPasswordException();
            }
            long timeout = getTokenTimeOutByValidity(Integer.parseInt(validity));
            String token = generateToken(user.getUserName(), user.getDisplayName(), user.getTenantId(),timeout );
            Device device = NetworkHelper.getDevice(req);
            TerminalHandler handler = TerminalHandler.getTerminal(req, device);
            LoginClientBean clientBean = new LoginClientBean(req, device, handler);
            clientBean.setUsername(user.getUserName());
            clientBean.setToken(token);
            clientBean.setValidity(Integer.parseInt(validity));
            clientBean.setUserId(user.getId());
            this.addLoginStatus(token, clientBean, timeout);
            this.writeToken2Cookie(res, token, Integer.parseInt(validity));

            domainResponse = CrossDomainResponse.create().callbackFuncName("callback").parameter("accessToken",token).parameter("url", render).parameter("status", "success");
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            domainResponse = CrossDomainResponse.error(e, "callback");
        }
        try {
            WebUtils.printAsString(res, domainResponse.createCrossDomainResponse());
            ;
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }
    private void writeToken2Cookie(HttpServletResponse res, String token, int validity) {
        try {
            if (StringUtils.isNotEmpty(token)) {
                Cookie tokenCookie = new Cookie("fine_auth_token", token);
                long timeout = validity == -2 ? 1209600000L : (long)validity;
                tokenCookie.setMaxAge((int)timeout);
                tokenCookie.setPath(ServerConfig.getInstance().getCookiePath());
                res.addCookie(tokenCookie);
                Cookie rememberLogin = new Cookie("fine_remember_login", String.valueOf(validity == -2 ? -2 : -1));
                rememberLogin.setMaxAge((int)timeout);
                rememberLogin.setPath(ServerConfig.getInstance().getCookiePath());
                res.addCookie(rememberLogin);
            } else {
                FineLoggerFactory.getLogger().error("empty token cannot save.");
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }

    }

    private void addLoginStatus(String token, LoginClientBean bean, long timeout) throws Exception {
        DecisionStatusService.loginStatusService().put(token, bean, new Converter<LoginClientBean>() {
            @Override
            public String[] createAlias(LoginClientBean bean) {
                return new String[]{bean.getUsername()};
            }
        }, timeout);
    }

    private long getTokenTimeOutByValidity(int validity) {
        return validity == -2 ? 1209600000L : FSConfig.getInstance().getLoginConfig().getLoginTimeout();
    }

    private String generateToken(String username, String displayName, String tenanId, long validity) {
        HashMap map = new HashMap();
        map.put("description", displayName);
        map.put("tenantId", tenanId);
        return JwtUtils.createDefaultJWT(username, map, validity);
    }
}
